#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-01-02 $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''

Really dump gzx ship plot builder.  This program will melt on files that are too large

@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: GPL
'''

import sys, os
from decimal import Decimal

import ais.ais_msg_1 as m1
import ais.binary as binary
from BitVector import BitVector

######################################################################
if __name__=='__main__':
	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options] file1.ais [file2.ais ...]",version="%prog "+__version__)

	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
                          help='Have the program be more talkative')

	parser.add_option('-o','--output-file',dest='outputFilename',default=None,
			  help='Name of the gzx to write [default: stdout]')
	(options,args) = parser.parse_args()
        verbose = options.verbose
        out=sys.stdout
        if None!=options.outputFilename: out=file(options.outputFilename,'w')

        ships={}  # dict of list of xyt tuples
        for filename in args:
            lineNum = 0
            for line in file(filename):
                lineNum += 1
                if lineNum%1000==0:
                    sys.stderr.write('line '+str(lineNum)+'\n')
                if line[3:6] not in ('VDM|VDO'): continue # Not an AIS VHF message

                try:
                    msgNum = int(binary.ais6tobitvec(line.split(',')[5][0]))
                except:
                    print 'line would not decode',line
                    continue

                bv = binary.ais6tobitvec(line.split(',')[5])
                if msgNum not in (1,2,3):
                    continue
                if len(bv) < 168:
                    print 'ERROR: skipping bad position message, line:',lineNum
                    print '  ',line,
                    print '   Got length',len(bv), 'expected', 168
                    continue

                # FIX: it had better be there!
                cg_timestamp = line.split(',')[-1].strip() # US Coast Guard time stamp.

                lon  = str(float(m1.decodePosition_longitude(bv)))
                lat  = str(float(m1.decodePosition_latitude(bv)))
                mmsi = m1.decodeUserID(bv)
                if mmsi not in ships: 
                    if verbose: print 
                    ships[mmsi]=[]
                ships[mmsi].append((lon,lat,cg_timestamp))
                #print ships

#        for ship in ships:
#            print '--------',ship,'-------'
#            print ships[ship]

        #out.write('<?xml version="1.0"?>\n')
        #out.write('\n')
        #out.write('<!-- Written by ais_gzx.py from noaadata.  Script by Kurt Schwehr -->\n')
        out.write('<GZ>\n')
        for mmsi in ships:
            if verbose: sys.stderr.write('writing ship: '+str(mmsi)+'\n')
            out.write('\t<Object type="Vessel" projection="Geographic" label="'+str(mmsi)+'">\n')
            data = ships[mmsi]
            for point in data:
                x=point[0]
                y=point[1]
                t=point[2]
                out.write('\t\t<Position position="'+x+' '+y+' 0" time="'+t+'"/>\n')
            out.write('\t</Object>\n')

        out.write('</GZ>\n')
